#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: pnp_device_history_info
short_description: Information module for Pnp Device History
description:
- Get all Pnp Device History.
- Returns history for a specific device. Serial number is a required parameter.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  serialNumber:
    description:
    - SerialNumber query parameter. Device Serial Number.
    type: str
  sort:
    description:
    - Sort query parameter. Comma seperated list of fields to sort on.
    elements: str
    type: list
  sortOrder:
    description:
    - SortOrder query parameter. Sort Order Ascending (asc) or Descending (des).
    type: str
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Device Onboarding (PnP) GetDeviceHistory
  description: Complete reference of the GetDeviceHistory API.
  link: https://developer.cisco.com/docs/dna-center/#!get-device-history
notes:
  - SDK Method used are
    device_onboarding_pnp.DeviceOnboardingPnp.get_device_history,

  - Paths used are
    get /dna/intent/api/v1/onboarding/pnp-device/history,

"""

EXAMPLES = r"""
- name: Get all Pnp Device History
  cisco.dnac.pnp_device_history_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    serialNumber: string
    sort: []
    sortOrder: string
  register: result

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "timestamp": 0,
          "details": "string",
          "historyTaskInfo": {
            "name": "string",
            "type": "string",
            "timeTaken": 0,
            "workItemList": [
              {
                "state": "string",
                "command": "string",
                "startTime": 0,
                "endTime": 0,
                "timeTaken": 0,
                "outputStr": "string"
              }
            ],
            "addnDetails": [
              {
                "key": "string",
                "value": "string"
              }
            ]
          },
          "errorFlag": true
        }
      ],
      "statusCode": 0
    }
"""



# -*- coding: utf-8 -*-
#
# Copyright (c) 2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


class ModuleDocFragment(object):

    BULK_OPERATIONS = r'''
options:
    bulk_operation_threshold:
        description:
            - Determines the threshold from when on bulk operations are used.
            - The default value 2 means that if 2 or more operations of a kind are planned,
              and the API supports bulk operations for this kind of operation, they will
              be used.
        type: int
        default: 2
'''

    RECORD_TRANSFORMATION = r'''
options:
    txt_transformation:
        description:
            - Determines how TXT entry values are converted between the API and this module's
              input and output.
            - The value V(api) means that values are returned from this module as they are returned
              from the API, and pushed to the API as they have been passed to this module. For
              idempotency checks, the input string will be compared to the strings returned by the
              API. The API might automatically transform some values, like splitting long values or
              adding quotes, which can cause problems with idempotency.
            - The value V(unquoted) automatically transforms values so that you can pass in unquoted
              values, and the module will return unquoted values. If you pass in quoted values, they
              will be double-quoted.
            - The value V(quoted) automatically transforms values so that you must use quoting for values
              that contain spaces, characters such as quotation marks and backslashes, and that are
              longer than 255 bytes. It also makes sure to return values from the API in a normalized
              encoding.
            - The default value, V(unquoted), ensures that you can work with values without having
              to care about how to correctly quote for DNS. Most users should use one of V(unquoted)
              or V(quoted), but not V(api).
            - B(Note:) the conversion code assumes UTF-8 encoding for values. If you need another
              encoding use O(txt_transformation=api) and handle the encoding yourself.
        type: str
        choices:
            - api
            - quoted
            - unquoted
        default: unquoted
    txt_character_encoding:
        description:
            - Whether to treat numeric escape sequences (V(\\xyz)) as octal or decimal numbers.
              This is only used when O(txt_transformation=quoted).
            - The current default is V(octal) which is deprecated. It will change to V(decimal) in
              community.dns 3.0.0. The value V(decimal) is compatible to L(RFC 1035, https://www.ietf.org/rfc/rfc1035.txt).
        type: str
        choices:
            - decimal
            - octal
        version_added: 2.5.0
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_vpc_subnet_info
version_added: 1.0.0
short_description: Gather information about ec2 VPC subnets in AWS
description:
    - Gather information about ec2 VPC subnets in AWS
author: "Rob White (@wimnat)"
options:
  subnet_ids:
    description:
      - A list of subnet IDs to gather information for.
    aliases: ['subnet_id']
    type: list
    elements: str
    default: []
  filters:
    description:
      - A dict of filters to apply. Each dict item consists of a filter key and a filter value.
        See U(https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeSubnets.html) for possible filters.
    type: dict
    default: {}
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

# Gather information about all VPC subnets
- amazon.aws.ec2_vpc_subnet_info:

# Gather information about a particular VPC subnet using ID
- amazon.aws.ec2_vpc_subnet_info:
    subnet_ids: subnet-00112233

# Gather information about any VPC subnet with a tag key Name and value Example
- amazon.aws.ec2_vpc_subnet_info:
    filters:
      "tag:Name": Example

# Gather information about any VPC subnet within VPC with ID vpc-abcdef00
- amazon.aws.ec2_vpc_subnet_info:
    filters:
      vpc-id: vpc-abcdef00

# Gather information about a set of VPC subnets, publicA, publicB and publicC within a
# VPC with ID vpc-abcdef00 and then use the jinja map function to return the
# subnet_ids as a list.

- amazon.aws.ec2_vpc_subnet_info:
    filters:
      vpc-id: vpc-abcdef00
      "tag:Name": "{{ item }}"
  loop:
    - publicA
    - publicB
    - publicC
  register: subnet_info

- set_fact:
    subnet_ids: "{{ subnet_info.results | sum(attribute='subnets', start=[]) | map(attribute='subnet_id') }}"
"""

RETURN = r"""
subnets:
    description: Returns an array of complex objects as described below.
    returned: success
    type: complex
    contains:
        subnet_id:
            description: The ID of the Subnet.
            returned: always
            type: str
        id:
            description: The ID of the Subnet (for backwards compatibility).
            returned: always
            type: str
        vpc_id:
            description: The ID of the VPC .
            returned: always
            type: str
        state:
            description: The state of the subnet.
            returned: always
            type: str
        tags:
            description: A dict of tags associated with the Subnet.
            returned: always
            type: dict
        map_public_ip_on_launch:
            description: True/False depending on attribute setting for public IP mapping.
            returned: always
            type: bool
        default_for_az:
            description: True if this is the default subnet for AZ.
            returned: always
            type: bool
        cidr_block:
            description: The IPv4 CIDR block assigned to the subnet.
            returned: always
            type: str
        available_ip_address_count:
            description: Count of available IPs in subnet.
            returned: always
            type: str
        availability_zone:
            description: The availability zone where the subnet exists.
            returned: always
            type: str
        assign_ipv6_address_on_creation:
            description: True/False depending on attribute setting for IPv6 address assignment.
            returned: always
            type: bool
        ipv6_cidr_block_association_set:
            description: An array of IPv6 cidr block association set information.
            returned: always
            type: complex
            contains:
                association_id:
                    description: The association ID
                    returned: always
                    type: str
                ipv6_cidr_block:
                    description: The IPv6 CIDR block that is associated with the subnet.
                    returned: always
                    type: str
                ipv6_cidr_block_state:
                    description: A hash/dict that contains a single item. The state of the cidr block association.
                    returned: always
                    type: dict
                    contains:
                        state:
                            description: The CIDR block association state.
                            returned: always
                            type: str
"""

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list


@AWSRetry.exponential_backoff()
def describe_subnets_with_backoff(connection, subnet_ids, filters):
    """
    Describe Subnets with AWSRetry backoff throttling support.

    connection  : boto3 client connection object
    subnet_ids  : list of subnet ids for which to gather information
    filters     : additional filters to apply to request
    """
    return connection.describe_subnets(SubnetIds=subnet_ids, Filters=filters)


def describe_subnets(connection, module):
    """
    Describe Subnets.

    module  : AnsibleAWSModule object
    connection  : boto3 client connection object
    """
    # collect parameters
    filters = ansible_dict_to_boto3_filter_list(module.params.get("filters"))
    subnet_ids = module.params.get("subnet_ids")

    if subnet_ids is None:
        # Set subnet_ids to empty list if it is None
        subnet_ids = []

    # init empty list for return vars
    subnet_info = list()

    # Get the basic VPC info
    try:
        response = describe_subnets_with_backoff(connection, subnet_ids, filters)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to describe subnets")

    for subnet in response["Subnets"]:
        # for backwards compatibility
        subnet["id"] = subnet["SubnetId"]
        subnet_info.append(camel_dict_to_snake_dict(subnet))
        # convert tag list to ansible dict
        subnet_info[-1]["tags"] = boto3_tag_list_to_ansible_dict(subnet.get("Tags", []))

    module.exit_json(subnets=subnet_info)


def main():
    argument_spec = dict(
        subnet_ids=dict(type="list", elements="str", default=[], aliases=["subnet_id"]),
        filters=dict(type="dict", default={}),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    connection = module.client("ec2")

    describe_subnets(connection, module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2019, Prasad Katti (@prasadkatti)
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: stepfunctions_state_machine_execution
version_added: 1.0.0

short_description: Start or stop execution of an AWS Step Functions state machine

description:
    - Start or stop execution of a state machine in AWS Step Functions.
    - Prior to release 5.0.0 this module was called C(community.aws.aws_step_functions_state_machine_execution).
      The usage did not change.

options:
    action:
        description: Desired action (C(start) or C(stop)) for a state machine execution.
        default: start
        choices: [ start, stop ]
        type: str
    name:
        description: Name of the execution.
        type: str
    execution_input:
        description: The JSON input data for the execution.
        type: json
        default: {}
    state_machine_arn:
        description: The ARN of the state machine that will be executed.
        type: str
    execution_arn:
        description: The ARN of the execution you wish to stop.
        type: str
    cause:
        description: A detailed explanation of the cause for stopping the execution.
        type: str
        default: ''
    error:
        description: The error code of the failure to pass in when stopping the execution.
        type: str
        default: ''

author:
    - Prasad Katti (@prasadkatti)

extends_documentation_fragment:
    - amazon.aws.common.modules
    - amazon.aws.region.modules
    - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Start an execution of a state machine
  community.aws.stepfunctions_state_machine_execution:
    name: an_execution_name
    execution_input: '{ "IsHelloWorldExample": true }'
    state_machine_arn: "arn:aws:states:us-west-2:123456789012:stateMachine:HelloWorldStateMachine"

- name: Stop an execution of a state machine
  community.aws.stepfunctions_state_machine_execution:
    action: stop
    execution_arn: "arn:aws:states:us-west-2:123456789012:execution:HelloWorldStateMachineCopy:a1e8e2b5-5dfe-d40e-d9e3-6201061047c8"
    cause: "cause of task failure"
    error: "error code of the failure"
"""

RETURN = r"""
execution_arn:
    description: ARN of the AWS Step Functions state machine execution.
    type: str
    returned: if action == start and changed == True
    sample: "arn:aws:states:us-west-2:123456789012:execution:HelloWorldStateMachineCopy:a1e8e2b5-5dfe-d40e-d9e3-6201061047c8"
start_date:
    description: The date the execution is started.
    type: str
    returned: if action == start and changed == True
    sample: "2019-11-02T22:39:49.071000-07:00"
stop_date:
    description: The date the execution is stopped.
    type: str
    returned: if action == stop
    sample: "2019-11-02T22:39:49.071000-07:00"
"""


try:
    import botocore
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def start_execution(module, sfn_client):
    """
    start_execution uses execution name to determine if a previous execution already exists.
    If an execution by the provided name exists, call client.start_execution will not be called.
    """

    state_machine_arn = module.params.get("state_machine_arn")
    name = module.params.get("name")
    execution_input = module.params.get("execution_input")

    try:
        # list_executions is eventually consistent
        page_iterators = sfn_client.get_paginator("list_executions").paginate(stateMachineArn=state_machine_arn)

        for execution in page_iterators.build_full_result()["executions"]:
            if name == execution["name"]:
                check_mode(module, msg="State machine execution already exists.", changed=False)
                module.exit_json(changed=False)

        check_mode(module, msg="State machine execution would be started.", changed=True)
        res_execution = sfn_client.start_execution(stateMachineArn=state_machine_arn, name=name, input=execution_input)
    except is_boto3_error_code("ExecutionAlreadyExists"):
        # this will never be executed anymore
        module.exit_json(changed=False)
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="Failed to start execution.")

    module.exit_json(changed=True, **camel_dict_to_snake_dict(res_execution))


def stop_execution(module, sfn_client):
    cause = module.params.get("cause")
    error = module.params.get("error")
    execution_arn = module.params.get("execution_arn")

    try:
        # describe_execution is eventually consistent
        execution_status = sfn_client.describe_execution(executionArn=execution_arn)["status"]
        if execution_status != "RUNNING":
            check_mode(module, msg="State machine execution is not running.", changed=False)
            module.exit_json(changed=False)

        check_mode(module, msg="State machine execution would be stopped.", changed=True)
        res = sfn_client.stop_execution(executionArn=execution_arn, cause=cause, error=error)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to stop execution.")

    module.exit_json(changed=True, **camel_dict_to_snake_dict(res))


def check_mode(module, msg="", changed=False):
    if module.check_mode:
        module.exit_json(changed=changed, output=msg)


def main():
    module_args = dict(
        action=dict(choices=["start", "stop"], default="start"),
        name=dict(type="str"),
        execution_input=dict(type="json", default={}),
        state_machine_arn=dict(type="str"),
        cause=dict(type="str", default=""),
        error=dict(type="str", default=""),
        execution_arn=dict(type="str"),
    )
    module = AnsibleAWSModule(
        argument_spec=module_args,
        required_if=[
            ("action", "start", ["name", "state_machine_arn"]),
            ("action", "stop", ["execution_arn"]),
        ],
        supports_check_mode=True,
    )

    sfn_client = module.client("stepfunctions")

    action = module.params.get("action")
    if action == "start":
        start_execution(module, sfn_client)
    else:
        stop_execution(module, sfn_client)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2020 Fred-Sun, (@Fred-Sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_privateendpoint
version_added: "1.8.0"
short_description: Manage Azure private endpoint
description:
    - Create, update or delete a private endpoint.
options:
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    location:
        description:
            - Valid Azure location. Defaults to location of the resource group.
        type: str
    name:
        description:
            - Name of the private endpoint.
        required: true
        type: str
    subnet:
        description:
            - The ID of the subnet from which the private IP will be allocated.
            - This parameter is required for create or update.
        type: dict
        suboptions:
            id:
                description:
                    - The ID of the subnet from which the private IP will be allocated.
                type: str
    private_link_service_connections:
        description:
            - A grouping of information about the connection to the remote resource.
            - This parameter is required for create or update.
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - The name of the resource that is unique within a resource group.
                type: str
            private_link_service_id:
                description:
                    - The resource id of the private endpoint to connect to.
                type: str
            group_ids:
                description:
                    - The ID(s) of the group(s) obtained from the remote resource that this private endpoint should connect to.
                type: list
                elements: str
    state:
        description:
            - State of the virtual network. Use C(present) to create or update and C(absent) to delete.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Fred-sun (@Fred-sun)

'''

EXAMPLES = '''
- name: Create private endpoint
  azure_rm_privateendpoint:
    name: testprivateendpoint
    resource_group: v-xisuRG
    private_link_service_connections:
      - name: Test_private_link_service
        private_link_service_id: /subscriptions/xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/privateLinkServices/testervice
    subnet:
      id: /subscriptions/xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/virtualNetworks/fredvnet/subnets/default
    tags:
      key1: value1
      key2: value2

- name: Delete private endpoint
  azure_rm_privateendpoint:
    name: testprivateendpoint
    resource_group: myResourceGroup
    state: absent
'''


RETURN = '''
state:
    description:
        - List of private endpoint dict with same format as M(azure.azcollection.azure_rm_privateendpoint) module paramter.
    returned: always
    type: complex
    contains:
            id:
                description:
                    - Resource ID of the private endpoint.
                sample: /subscriptions/xxx-xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/privateEndpoints/testprivateendpoint
                returned: always
                type: str
            etag:
                description:
                    -  A unique read-only string that changes whenever the resource is updated.
                sample: 'W/\"20803842-7d51-46b2-a790-ded8971b4d8a'
                returned: always
                type: str
            network_interfaces:
                description:
                    - List ID of the network interfaces.
                returned: always
                type: list
                sample:  ["/subscriptions/xxx-xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/networkInterfaces/fredprivateendpoint002.nic"]
            location:
                description:
                    - Valid Azure location.
                returned: always
                type: str
                sample: eastus
            tags:
                description:
                    - Tags assigned to the resource. Dictionary of string:string pairs.
                returned: always
                type: dict
                sample: { "tag1": "abc" }
            provisioning_state:
                description:
                    - Provisioning state of the resource.
                returned: always
                sample: Succeeded
                type: str
            name:
                description:
                    - Name of the private endpoint.
                returned: always
                type: str
                sample: estprivateendpoint
            subnets_id:
                description:
                    - Subnets associated with the virtual network.
                returned: always
                type: str
                sample: "/subscriptions/xxx-xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/virtualNetworks/fredtestRG-vnet/subnets/default"
            private_link_service_connections:
                description:
                    - The resource id of the private endpoint to connect.
                returned: always
                type: list
                sample: ["/subscriptions/xxx/resourceGroups/myRG/providers/Microsoft.Network/privateEndpoints/point/privateLinkServiceConnections/point",]
            type:
                description:
                    - Resource type.
                returned: always
                type: str
                sample: Microsoft.Network/privateEndpoints
'''

try:
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt


network_interfaces_spec = dict(
    id=dict(type='str')
)


private_service_connection_spec = dict(
    name=dict(type='str'),
    private_link_service_id=dict(type='str'),
    group_ids=dict(type='list', elements='str')
)


subnet_spec = dict(
    id=dict(type='str')
)


class Actions:
    NoAction, Create, Update, Delete = range(4)


class AzureRMPrivateEndpoint(AzureRMModuleBaseExt):

    def __init__(self):

        self.module_arg_spec = dict(
            resource_group=dict(type='str', required=True),
            name=dict(type='str', required=True),
            state=dict(type='str', default='present', choices=['present', 'absent']),
            location=dict(type='str'),
            subnet=dict(type='dict', options=subnet_spec),
            private_link_service_connections=dict(type='list', elements='dict', options=private_service_connection_spec),
        )

        self.resource_group = None
        self.name = None
        self.state = None
        self.location = None
        self.body = {}
        self.tags = None

        self.results = dict(
            changed=False,
            state=dict()
        )
        self.to_do = Actions.NoAction

        super(AzureRMPrivateEndpoint, self).__init__(self.module_arg_spec,
                                                     supports_tags=True,
                                                     supports_check_mode=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                self.body[key] = kwargs[key]

        self.inflate_parameters(self.module_arg_spec, self.body, 0)

        resource_group = self.get_resource_group(self.resource_group)
        if not self.location:
            # Set default location
            self.location = resource_group.location
        self.body['location'] = self.location
        self.body['tags'] = self.tags

        self.log('Fetching private endpoint {0}'.format(self.name))
        old_response = self.get_resource()

        if old_response is None:
            if self.state == "present":
                self.to_do = Actions.Create
        else:
            if self.state == 'absent':
                self.to_do = Actions.Delete
            else:
                update_tags, newtags = self.update_tags(old_response.get('tags', {}))
                if update_tags:
                    self.body['tags'] = newtags
                    self.to_do = Actions.Update

        if (self.to_do == Actions.Create) or (self.to_do == Actions.Update):
            self.results['changed'] = True
            if self.check_mode:
                return self.results
            response = self.create_update_resource_private_endpoint(self.body)
        elif self.to_do == Actions.Delete:
            self.results['changed'] = True
            if self.check_mode:
                return self.results
            response = self.delete_private_endpoint()
        else:
            self.results['changed'] = False
            response = old_response
        if response is not None:
            self.results['state'] = response
        return self.results

    def create_update_resource_private_endpoint(self, privateendpoint):
        try:
            poller = self.network_client.private_endpoints.begin_create_or_update(resource_group_name=self.resource_group,
                                                                                  private_endpoint_name=self.name, parameters=privateendpoint)
            new_privateendpoint = self.get_poller_result(poller)
        except Exception as exc:
            self.fail("Error creating or updating private endpoint {0} - {1}".format(self.name, str(exc)))

        return self.private_endpoints_to_dict(new_privateendpoint)

    def delete_private_endpoint(self):
        try:
            poller = self.network_client.private_endpoints.begin_delete(self.resource_group, self.name)
            result = self.get_poller_result(poller)
        except Exception as exc:
            self.fail("Error deleting private endpoint {0} - {1}".format(self.name, str(exc)))
        return result

    def get_resource(self):
        found = False
        try:
            private_endpoint = self.network_client.private_endpoints.get(self.resource_group, self.name)
            results = self.private_endpoints_to_dict(private_endpoint)
            found = True
            self.log("Response : {0}".format(results))
        except ResourceNotFoundError:
            self.log("Did not find the private endpoint resource")
        if found is True:
            return results
        else:
            return None

    def private_endpoints_to_dict(self, privateendpoint):
        results = dict(
            id=privateendpoint.id,
            name=privateendpoint.name,
            location=privateendpoint.location,
            tags=privateendpoint.tags,
            provisioning_state=privateendpoint.provisioning_state,
            type=privateendpoint.type,
            etag=privateendpoint.etag,
            subnet=dict(id=privateendpoint.subnet.id)
        )
        if privateendpoint.network_interfaces and len(privateendpoint.network_interfaces) > 0:
            results['network_interfaces'] = []
            for interface in privateendpoint.network_interfaces:
                results['network_interfaces'].append(interface.id)
        if privateendpoint.private_link_service_connections and len(privateendpoint.private_link_service_connections) > 0:
            results['private_link_service_connections'] = []
            for connections in privateendpoint.private_link_service_connections:
                results['private_link_service_connections'].append(dict(private_link_service_id=connections.private_link_service_id, name=connections.name))

        return results


def main():
    AzureRMPrivateEndpoint()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#############################################

"""
The module file for nxos_ospfv3
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: nxos_ospfv3
short_description: OSPFv3 resource module
description:
- This module manages OSPFv3 configuration on devices running Cisco NX-OS.
version_added: 1.2.0
notes:
- Tested against NX-OS 7.0(3)I5(1).
- Unsupported for Cisco MDS
- This module works with connection C(network_cli) and C(httpapi).
author: Nilashish Chakraborty (@NilashishC)
options:
  running_config:
    description:
    - This option is used only with state I(parsed).
    - The value of this option should be the output received from the NX-OS device
      by executing the command B(show running-config | section "^router ospfv3").
    - The state I(parsed) reads the configuration from C(running_config) option and
      transforms it into Ansible structured data as per the resource module's argspec
      and the value is then returned in the I(parsed) key within the result.
    type: str
  config:
    description: A list of OSPFv3 process configuration.
    type: dict
    suboptions:
      processes:
        description:
        - A list of OSPFv3 instances' configurations.
        type: list
        elements: dict
        suboptions:
          address_family:
            description:
            - IPv6 unicast address-family OSPFv3 settings.
            type: dict
            suboptions:
              afi:
                description:
                - Configure OSPFv3 settings under IPv6 address-family.
                type: str
                choices: ['ipv6']
              safi:
                description:
                - Configure OSPFv3 settings under IPv6 unicast address-family.
                type: str
                choices: ['unicast']
              areas:
                description:
                - Configure properties of OSPF Areas under address-family.
                type: list
                elements: dict
                suboptions:
                  area_id:
                    description:
                    - The Area ID in IP Address format.
                    type: str
                    required: True
                  default_cost:
                    description:
                    - Specify the default cost.
                    type: int
                  filter_list:
                    description:
                    - Filter prefixes between OSPF areas.
                    type: list
                    elements: dict
                    suboptions:
                      route_map:
                        description:
                        - The Route-map name.
                        type: str
                        required: True
                      direction:
                        description:
                        - The direction to apply the route map.
                        type: str
                        choices: [in, out]
                        required: True
                  ranges:
                    description:
                    - Configure an address range for the area.
                    type: list
                    elements: dict
                    suboptions:
                      prefix:
                        description:
                        - IP in Prefix format (x.x.x.x/len)
                        type: str
                        required: True
                      cost:
                        description:
                        - Cost to use for the range.
                        type: int
                      not_advertise:
                        description:
                        - Suppress advertising the specified range.
                        type: bool
              default_information:
                description:
                - Control distribution of default routes.
                type: dict
                suboptions:
                  originate:
                    description:
                    - Distribute a default route.
                    type: dict
                    suboptions:
                      set:
                        description:
                        - Enable distribution of default route.
                        type: bool
                      always:
                        description:
                        - Always advertise a default route.
                        type: bool
                      route_map:
                        description:
                        - Policy to control distribution of default routes
                        type: str
              distance:
                description:
                - Configure the OSPF administrative distance.
                type: int
              maximum_paths:
                description:
                - Maximum paths per destination.
                type: int
              redistribute:
                description:
                - Redistribute information from another routing protocol.
                type: list
                elements: dict
                suboptions:
                  protocol:
                    description:
                    - The name of the protocol.
                    type: str
                    choices: [bgp, direct, eigrp, isis, lisp, ospfv3, rip, static]
                    required: True
                  id:
                    description:
                    - The identifier for the protocol specified.
                    type: str
                  route_map:
                    description:
                    - The route map policy to constrain redistribution.
                    type: str
                    required: True
              summary_address:
                description:
                - Configure route summarization for redistribution.
                type: list
                elements: dict
                suboptions:
                  prefix:
                    description:
                    - IPv6 prefix format 'xxxx:xxxx/ml', 'xxxx:xxxx::/ml' or 'xxxx::xx/128'
                    type: str
                    required: True
                  not_advertise:
                    description:
                    - Suppress advertising the specified summary.
                    type: bool
                  tag:
                    description:
                    - A 32-bit tag value.
                    type: int
              table_map:
                description:
                - Policy for filtering/modifying OSPF routes before sending them to
                  RIB.
                type: dict
                suboptions:
                  name:
                    description:
                    - The Route Map name.
                    type: str
                    required: True
                  filter:
                    description:
                    - Block the OSPF routes from being sent to RIB.
                    type: bool
              timers:
                description:
                - Configure timer related constants.
                type: dict
                suboptions:
                  throttle:
                    description:
                    - Configure throttle related constants.
                    type: dict
                    suboptions:
                      spf:
                        description:
                        - Set OSPF SPF timers.
                        type: dict
                        suboptions:
                          initial_spf_delay:
                            description:
                            - Initial SPF schedule delay in milliseconds.
                            type: int
                          min_hold_time:
                            description:
                            - Minimum hold time between SPF calculations.
                            type: int
                          max_wait_time:
                            description:
                            - Maximum wait time between SPF calculations.
                            type: int
          areas:
            description:
            - Configure properties of OSPF Areas.
            type: list
            elements: dict
            suboptions:
              area_id:
                description:
                - The Area ID in IP Address format.
                type: str
                required: True
              nssa:
                description:
                - NSSA settings for the area.
                type: dict
                suboptions:
                  set:
                    description:
                    - Configure area as NSSA.
                    type: bool
                  default_information_originate:
                    description:
                    - Originate Type-7 default LSA into NSSA area.
                    type: bool
                  no_redistribution:
                    description:
                    - Do not send redistributed LSAs into NSSA area.
                    type: bool
                  no_summary:
                    description:
                    - Do not send summary LSAs into NSSA area.
                    type: bool
                  route_map:
                    description:
                    - Policy to control distribution of default route.
                    type: str
                  translate:
                    description:
                    - Translate LSA.
                    type: dict
                    suboptions:
                      type7:
                        description:
                        - Translate from Type 7 to Type 5.
                        type: dict
                        suboptions:
                          always:
                            description:
                            - Always translate LSAs
                            type: bool
                          never:
                            description:
                            - Never translate LSAs
                            type: bool
                          supress_fa:
                            description:
                            - Suppress forwarding address in translated LSAs.
                            type: bool
              stub:
                description:
                - Settings for configuring the area as a stub.
                type: dict
                suboptions:
                  set:
                    description:
                    - Configure the area as a stub.
                    type: bool
                  no_summary:
                    description:
                    - Prevent ABR from sending summary LSAs into stub area.
                    type: bool
          auto_cost:
            description:
            - Calculate OSPF cost according to bandwidth.
            type: dict
            suboptions:
              reference_bandwidth:
                description:
                - Reference bandwidth used to assign OSPF cost.
                type: int
                required: True
              unit:
                description:
                - Specify in which unit the reference bandwidth is specified.
                type: str
                required: True
                choices: [Gbps, Mbps]
          flush_routes:
            description:
            - Flush routes on a non-graceful controlled restart.
            type: bool
          graceful_restart:
            description:
            - Configure graceful restart.
            type: dict
            suboptions:
              set:
                description:
                - Enable graceful-restart.
                type: bool
              grace_period:
                description:
                - Configure maximum interval to restart gracefully.
                type: int
              helper_disable:
                description:
                - Enable/Disable helper mode.
                type: bool
              planned_only:
                description:
                - Enable graceful restart only for a planned restart
                type: bool
          isolate:
            description:
            - Isolate this router from OSPF perspective.
            type: bool
          log_adjacency_changes:
            description:
            - Log changes in adjacency state.
            type: dict
            suboptions:
              log:
                description:
                - Enable/disable logging changes in adjacency state.
                type: bool
              detail:
                description:
                - Notify all state changes.
                type: bool
          max_lsa:
            description:
            - Feature to limit the number of non-self-originated LSAs.
            type: dict
            suboptions:
              max_non_self_generated_lsa:
                description:
                - Set the maximum number of non self-generated LSAs.
                type: int
                required: True
              threshold:
                description:
                - Threshold value (%) at which to generate a warning message.
                type: int
              ignore_count:
                description:
                - Set count on how many times adjacencies can be suppressed.
                type: int
              ignore_time:
                description:
                - Set time during which all adjacencies are suppressed.
                type: int
              reset_time:
                description:
                - Set number of minutes after which ignore-count is reset to zero.
                type: int
              warning_only:
                description:
                - Log a warning message when limit is exceeded.
                type: bool
          max_metric:
            description:
            - Maximize the cost metric.
            type: dict
            suboptions:
              router_lsa:
                description:
                - Router LSA configuration.
                type: dict
                suboptions:
                  set:
                    description:
                    - Set router-lsa attribute.
                    type: bool
                  external_lsa:
                    description:
                    - External LSA configuration.
                    type: dict
                    suboptions:
                      set:
                        description:
                        - Set external-lsa attribute.
                        type: bool
                      max_metric_value:
                        description:
                        - Set max metric value for external LSAs.
                        type: int
                  stub_prefix_lsa:
                    description:
                    - Advertise Max metric for Stub links as well.
                    type: bool
                  on_startup:
                    description:
                    - Effective only at startup.
                    type: dict
                    suboptions:
                      set:
                        description:
                        - Set on-startup attribute.
                        type: bool
                      wait_period:
                        description:
                        - Wait period in seconds after startup.
                        type: int
                      wait_for_bgp_asn:
                        description:
                        - ASN of BGP to wait for.
                        type: int
                  inter_area_prefix_lsa:
                    description:
                    - Inter-area-prefix LSAs configuration.
                    type: dict
                    suboptions:
                      set:
                        description:
                        - Set summary-lsa attribute.
                        type: bool
                      max_metric_value:
                        description:
                        - Max metric value for summary LSAs.
                        type: int
          name_lookup:
            description:
            - Display OSPF router ids as DNS names.
            type: bool
          passive_interface:
            description:
            - Suppress routing updates on the interface.
            type: dict
            suboptions:
              default:
                description:
                - Interfaces passive by default.
                type: bool
          process_id:
            description:
            - The OSPF process tag.
            type: str
            required: True
          router_id:
            description:
            - Set OSPF process router-id.
            type: str
          shutdown:
            description:
            - Shutdown the OSPF protocol instance.
            type: bool
          timers:
            description:
            - Configure timer related constants.
            type: dict
            suboptions:
              lsa_arrival:
                description:
                - Mimimum interval between arrival of a LSA.
                type: int
              lsa_group_pacing:
                description:
                - LSA group refresh/maxage interval.
                type: int
              throttle:
                description:
                - Configure throttle related constants.
                type: dict
                suboptions:
                  lsa:
                    description:
                    - Set rate-limiting for LSA generation.
                    type: dict
                    suboptions:
                      start_interval:
                        description:
                        - The start interval.
                        type: int
                      hold_interval:
                        description:
                        - The hold interval.
                        type: int
                      max_interval:
                        description:
                        - The max interval.
                        type: int
          vrfs:
            description:
            - Configure VRF specific OSPF settings.
            type: list
            elements: dict
            suboptions:
              areas:
                description:
                - Configure properties of OSPF Areas.
                type: list
                elements: dict
                suboptions:
                  area_id:
                    description:
                    - The Area ID in IP Address format.
                    type: str
                    required: True
                  nssa:
                    description:
                    - NSSA settings for the area.
                    type: dict
                    suboptions:
                      set:
                        description:
                        - Configure area as NSSA.
                        type: bool
                      default_information_originate:
                        description:
                        - Originate Type-7 default LSA into NSSA area.
                        type: bool
                      no_redistribution:
                        description:
                        - Do not send redistributed LSAs into NSSA area.
                        type: bool
                      no_summary:
                        description:
                        - Do not send summary LSAs into NSSA area.
                        type: bool
                      route_map:
                        description:
                        - Policy to control distribution of default route.
                        type: str
                      translate:
                        description:
                        - Translate LSA.
                        type: dict
                        suboptions:
                          type7:
                            description:
                            - Translate from Type 7 to Type 5.
                            type: dict
                            suboptions:
                              always:
                                description:
                                - Always translate LSAs
                                type: bool
                              never:
                                description:
                                - Never translate LSAs
                                type: bool
                              supress_fa:
                                description:
                                - Suppress forwarding address in translated LSAs.
                                type: bool
                  stub:
                    description:
                    - Settings for configuring the area as a stub.
                    type: dict
                    suboptions:
                      set:
                        description:
                        - Configure the area as a stub.
                        type: bool
                      no_summary:
                        description:
                        - Prevent ABR from sending summary LSAs into stub area.
                        type: bool
              auto_cost:
                description:
                - Calculate OSPF cost according to bandwidth.
                type: dict
                suboptions:
                  reference_bandwidth:
                    description:
                    - Reference bandwidth used to assign OSPF cost.
                    type: int
                    required: True
                  unit:
                    description:
                    - Specify in which unit the reference bandwidth is specified.
                    type: str
                    required: True
                    choices: [Gbps, Mbps]
              graceful_restart:
                description:
                - Configure graceful restart.
                type: dict
                suboptions:
                  set:
                    description:
                    - Enable graceful-restart.
                    type: bool
                  grace_period:
                    description:
                    - Configure maximum interval to restart gracefully.
                    type: int
                  helper_disable:
                    description:
                    - Enable/Disable helper mode.
                    type: bool
                  planned_only:
                    description:
                    - Enable graceful restart only for a planned restart
                    type: bool
              log_adjacency_changes:
                description:
                - Log changes in adjacency state.
                type: dict
                suboptions:
                  log:
                    description:
                    - Enable/disable logging changes in adjacency state.
                    type: bool
                  detail:
                    description:
                    - Notify all state changes.
                    type: bool
              max_lsa:
                description:
                - Feature to limit the number of non-self-originated LSAs.
                type: dict
                suboptions:
                  max_non_self_generated_lsa:
                    description:
                    - Set the maximum number of non self-generated LSAs.
                    type: int
                    required: True
                  threshold:
                    description:
                    - Threshold value (%) at which to generate a warning message.
                    type: int
                  ignore_count:
                    description:
                    - Set count on how many times adjacencies can be suppressed.
                    type: int
                  ignore_time:
                    description:
                    - Set time during which all adjacencies are suppressed.
                    type: int
                  reset_time:
                    description:
                    - Set number of minutes after which ignore-count is reset to zero.
                    type: int
                  warning_only:
                    description:
                    - Log a warning message when limit is exceeded.
                    type: bool
              max_metric:
                description:
                - Maximize the cost metric.
                type: dict
                suboptions:
                  router_lsa:
                    description:
                    - Router LSA configuration.
                    type: dict
                    suboptions:
                      set:
                        description:
                        - Set router-lsa attribute.
                        type: bool
                      external_lsa:
                        description:
                        - External LSA configuration.
                        type: dict
                        suboptions:
                          set:
                            description:
                            - Set external-lsa attribute.
                            type: bool
                          max_metric_value:
                            description:
                            - Set max metric value for external LSAs.
                            type: int
                      stub_prefix_lsa:
                        description:
                        - Advertise Max metric for Stub links as well.
                        type: bool
                      on_startup:
                        description:
                        - Effective only at startup.
                        type: dict
                        suboptions:
                          set:
                            description:
                            - Set on-startup attribute.
                            type: bool
                          wait_period:
                            description:
                            - Wait period in seconds after startup.
                            type: int
                          wait_for_bgp_asn:
                            description:
                            - ASN of BGP to wait for.
                            type: int
                      inter_area_prefix_lsa:
                        description:
                        - Inter-area-prefix LSAs configuration.
                        type: dict
                        suboptions:
                          set:
                            description:
                            - Set summary-lsa attribute.
                            type: bool
                          max_metric_value:
                            description:
                            - Max metric value for summary LSAs.
                            type: int
              name_lookup:
                description:
                - Display OSPF router ids as DNS names.
                type: bool
              passive_interface:
                description:
                - Suppress routing updates on the interface.
                type: dict
                suboptions:
                  default:
                    description:
                    - Interfaces passive by default.
                    type: bool
              router_id:
                description:
                - Set OSPF process router-id.
                type: str
              shutdown:
                description:
                - Shutdown the OSPF protocol instance.
                type: bool
              timers:
                description:
                - Configure timer related constants.
                type: dict
                suboptions:
                  lsa_arrival:
                    description:
                    - Mimimum interval between arrival of a LSA.
                    type: int
                  lsa_group_pacing:
                    description:
                    - LSA group refresh/maxage interval.
                    type: int
                  throttle:
                    description:
                    - Configure throttle related constants.
                    type: dict
                    suboptions:
                      lsa:
                        description:
                        - Set rate-limiting for LSA generation.
                        type: dict
                        suboptions:
                          start_interval:
                            description:
                            - The start interval.
                            type: int
                          hold_interval:
                            description:
                            - The hold interval.
                            type: int
                          max_interval:
                            description:
                            - The max interval.
                            type: int
              vrf:
                description:
                - Name/Identifier of the VRF.
                type: str
                required: True
  state:
    description:
    - The state the configuration should be left in.
    type: str
    choices:
    - merged
    - replaced
    - overridden
    - deleted
    - gathered
    - parsed
    - rendered
    default: merged
"""
EXAMPLES = """
# Using merged

# Before state:
# -------------
# nxos-9k-rdo# sh running-config | section "^router ospfv3"
# nxos-9k-rdo#

- name: Merge the provided configuration with the existing running configuration
  cisco.nxos.nxos_ospfv3:
    config:
      processes:
      - process_id: 100
        router_id: 203.0.113.20
      - process_id: 102
        router_id: 198.51.100.1
        address_family:
          afi: ipv6
          safi: unicast
          areas:
          - area_id: 0.0.0.100
            filter_list:
            - route_map: rmap_1
              direction: in
            - route_map: rmap_2
              direction: out
            ranges:
            - prefix: 2001:db2::/32
              not_advertise: true
            - prefix: 2001:db3::/32
              cost: 120
          redistribute:
          - protocol: eigrp
            id: 120
            route_map: rmap_1
          - protocol: direct
            route_map: ospf102-direct-connect
        vrfs:
        - vrf: zone1
          router_id: 198.51.100.129
          areas:
          - area_id: 0.0.0.102
            nssa:
              default_information_originate: true
              no_summary: true
          - area_id: 0.0.0.103
            nssa:
              no_summary: true
              translate:
                type7:
                  always: true
        - vrf: zone2
          auto_cost:
            reference_bandwidth: 45
            unit: Gbps
    state: merged

# Task output:
# ------------
# before: {}
#
# commands:
#  - router ospf 102
#  - router-id 198.51.100.1
#  - address-family ipv6 unicast
#  - redistribute eigrp 120 route-map rmap_1
#  - redistribute direct route-map ospf102-direct-connect
#  - area 0.0.0.100 filter-list route-map rmap_1 in
#  - area 0.0.0.100 filter-list route-map rmap_2 out
#  - area 0.0.0.100 range 2001:db2::/32 not-advertise
#  - area 0.0.0.100 range 2001:db3::/32 cost 120
#  - vrf zone1
#  - router-id 198.51.100.129
#  - area 0.0.0.102 nssa no-summary default-information-originate
#  - area 0.0.0.103 nssa no-summary
#  - area 0.0.0.103 nssa translate type7 always
#  - vrf zone2
#  - auto-cost reference-bandwidth 45 Gbps
#  - router ospf 100
#  - router-id 203.0.113.20
#
# after:
#    processes:
#    - process_id: "100"
#      router_id: 203.0.113.20
#    - address_family:
#        afi: ipv4
#        safi: unicast
#        areas:
#        - area_id: 0.0.0.100
#          filter_list:
#          - direction: out
#            route_map: rmap_2
#          - direction: in
#            route_map: rmap_1
#          ranges:
#          - not_advertise: true
#            prefix: 2001:db2::/32
#          - cost: 120
#            prefix: 2001:db3::/32
#        redistribute:
#        - protocol: direct
#          route_map: ospf102-direct-connect
#        - id: "120"
#          protocol: eigrp
#          route_map: rmap_1
#      process_id: "102"
#      router_id: 198.51.100.1
#      vrfs:
#      - areas:
#        - area_id: 0.0.0.102
#          nssa:
#            default_information_originate: true
#            no_summary: true
#        - area_id: 0.0.0.103
#          nssa:
#            no_summary: true
#            translate:
#              type7:
#                always: true
#        router_id: 198.51.100.129
#        vrf: zone1
#      - auto_cost:
#          reference_bandwidth: 45
#          unit: Gbps
#        vrf: zone2
#

# After state:
# ------------
# nxos-9k-rdo# sh running-config | section "^router ospfv3"
# router ospfv3 100
#   router-id 203.0.113.20
# router ospfv3 102
#   router-id 198.51.100.1
#   address-family ipv6 unicast
#     redistribute direct route-map ospf102-direct-connect
#     redistribute eigrp 120 route-map rmap_1
#     area 0.0.0.100 filter-list route-map rmap_2 out
#     area 0.0.0.100 filter-list route-map rmap_1 in
#     area 0.0.0.100 range 2001:db2::/32 not-advertise
#     area 0.0.0.100 range 2001:db3::/32 cost 120
#   vrf zone1
#     router-id 198.51.100.129
#     area 0.0.0.102 nssa no-summary default-information-originate
#     area 0.0.0.103 nssa no-summary
#     area 0.0.0.103 nssa translate type7 always
#   vrf zone2
#     auto-cost reference-bandwidth 45 Gbps

# Using replaced

# Before state:
# -------------
# nxos-9k-rdo# sh running-config | section "^router ospfv3"
# router ospfv3 100
#   router-id 203.0.113.20
# router ospfv3 102
#   router-id 198.51.100.1
#   address-family upv6 unicast
#     redistribute direct route-map ospf102-direct-connect
#     redistribute eigrp 120 route-map rmap_1
#     area 0.0.0.100 filter-list route-map rmap_2 out
#     area 0.0.0.100 filter-list route-map rmap_1 in
#     area 0.0.0.100 range 2001:db2::/32 not-advertise
#     area 0.0.0.100 range 2001:db3::/32 cost 120
#   vrf zone1
#     router-id 198.51.100.129
#     area 0.0.0.102 nssa no-summary default-information-originate
#     area 0.0.0.103 nssa no-summary
#     area 0.0.0.103 nssa translate type7 always
#   vrf zone2
#     auto-cost reference-bandwidth 45 Gbps

- name: Replace device configurations of listed OSPFv3 processes with provided configurations
  cisco.nxos.nxos_ospfv3:
    config:
      processes:
      - process_id: 102
        router_id: 198.51.100.1
        address_family:
          afi: ipv6
          safi: unicast
          areas:
          - area_id: 0.0.0.100
            filter_list:
            - route_map: rmap_8
              direction: in
            ranges:
            - not_advertise: true
              prefix: 2001:db2::/32
          redistribute:
          - protocol: eigrp
            id: 130
            route_map: rmap_1
          - protocol: direct
            route_map: ospf102-direct-connect
        vrfs:
        - vrf: zone1
          router_id: 198.51.100.129
          areas:
          - area_id: 0.0.0.102
            nssa:
              default_information_originate: True
              no_summary: True
    state: replaced

# Task output:
# ------------
# before:
#    processes:
#    - process_id: "100"
#      router_id: 203.0.113.20
#    - address_family:
#        afi: ipv4
#        safi: unicast
#        areas:
#        - area_id: 0.0.0.100
#          filter_list:
#          - direction: out
#            route_map: rmap_2
#          - direction: in
#            route_map: rmap_1
#          ranges:
#          - not_advertise: true
#            prefix: 2001:db2::/32
#          - cost: 120
#            prefix: 2001:db3::/32
#        redistribute:
#        - protocol: direct
#          route_map: ospf102-direct-connect
#        - id: "120"
#          protocol: eigrp
#          route_map: rmap_1
#      process_id: "102"
#      router_id: 198.51.100.1
#      vrfs:
#      - areas:
#        - area_id: 0.0.0.102
#          nssa:
#            default_information_originate: true
#            no_summary: true
#        - area_id: 0.0.0.103
#          nssa:
#            no_summary: true
#            translate:
#              type7:
#                always: true
#        router_id: 198.51.100.129
#        vrf: zone1
#      - auto_cost:
#          reference_bandwidth: 45
#          unit: Gbps
#        vrf: zone2
#
#  commands:
#  - router ospf 102
#  - address-family ipv6 unicast
#  - redistribute eigrp 130 route-map rmap_1
#  - no redistribute eigrp 120 route-map rmap_1
#  - area 0.0.0.100 filter-list route-map rmap_8 in
#  - no area 0.0.0.100 filter-list route-map rmap_2 out
#  - no area 0.0.0.100 range 2001:db3::/32
#  - vrf zone1
#  - no area 0.0.0.103 nssa
#  - no area 0.0.0.103 nssa translate type7 always
#  - no vrf zone2
#
# after:
#    processes:
#    - process_id: "100"
#      router_id: 203.0.113.20
#    - address_family:
#        afi: ipv6
#        safi: unicast
#        areas:
#        - area_id: 0.0.0.100
#          filter_list:
#          - direction: in
#            route_map: rmap_8
#          ranges:
#          - not_advertise: true
#            prefix: 2001:db2::/32
#        redistribute:
#        - protocol: direct
#          route_map: ospf102-direct-connect
#        - id: "130"
#          protocol: eigrp
#          route_map: rmap_1
#      process_id: "102"
#      router_id: 198.51.100.1
#      vrfs:
#      - areas:
#        - area_id: 0.0.0.102
#          nssa:
#            default_information_originate: true
#            no_summary: true
#        router_id: 198.51.100.129
#        vrf: zone1

# After state:
# ------------
# nxos-9k-rdo# sh running-config | section "^router ospfv3"
# router ospfv3 100
#   router-id 203.0.113.20
# router ospfv3 102
#   router-id 198.51.100.1
#   address-family ipv6 unicast
#     redistribute direct route-map ospf102-direct-connect
#     redistribute eigrp 130 route-map rmap_1
#     area 0.0.0.100 filter-list route-map rmap_8 in
#     area 0.0.0.100 range 198.51.100.64/27 not-advertise
#   vrf zone1
#     router-id 198.51.100.129
#     area 0.0.0.102 nssa no-summary default-information-originate

# Using overridden

# Before state:
# -------------
# nxos-9k-rdo# sh running-config | section "^router ospfv3"
# router ospfv3 100
#   router-id 203.0.113.20
# router ospfv3 102
#   router-id 198.51.100.1
#   address-family ipv6 unicast
#     redistribute direct route-map ospf102-direct-connect
#     redistribute eigrp 120 route-map rmap_1
#     area 0.0.0.100 filter-list route-map rmap_2 out
#     area 0.0.0.100 filter-list route-map rmap_1 in
#     area 0.0.0.100 range 2001:db2::/32 not-advertise
#     area 0.0.0.100 range 2001:db3::/32 cost 120
#   vrf zone1
#     router-id 198.51.100.129
#     area 0.0.0.102 nssa no-summary default-information-originate
#     area 0.0.0.103 nssa no-summary
#     area 0.0.0.103 nssa translate type7 always
#   vrf zone2
#     auto-cost reference-bandwidth 45 Gbps

- name: Override all OSPFv3 configuration with provided configuration
  cisco.nxos.nxos_ospfv3:
    config:
      processes:
      - process_id: 104
        router_id: 203.0.113.20
      - process_id: 102
        router_id: 198.51.100.1
        shutdown: true
    state: overridden

# Task output:
# ------------
# before:
#    processes:
#    - process_id: "100"
#      router_id: 203.0.113.20
#    - address_family:
#        afi: ipv4
#        safi: unicast
#        areas:
#        - area_id: 0.0.0.100
#          filter_list:
#          - direction: out
#            route_map: rmap_2
#          - direction: in
#            route_map: rmap_1
#          ranges:
#          - not_advertise: true
#            prefix: 2001:db2::/32
#          - cost: 120
#            prefix: 2001:db3::/32
#        redistribute:
#        - protocol: direct
#          route_map: ospf102-direct-connect
#        - id: "120"
#          protocol: eigrp
#          route_map: rmap_1
#      process_id: "102"
#      router_id: 198.51.100.1
#      vrfs:
#      - areas:
#        - area_id: 0.0.0.102
#          nssa:
#            default_information_originate: true
#            no_summary: true
#        - area_id: 0.0.0.103
#          nssa:
#            no_summary: true
#            translate:
#              type7:
#                always: true
#        router_id: 198.51.100.129
#        vrf: zone1
#      - auto_cost:
#          reference_bandwidth: 45
#          unit: Gbps
#        vrf: zone2
#
# commands:
#  - no router ospfv3 100
#  - router ospfv3 104
#  - router-id 203.0.113.20
#  - router ospfv3 102
#  - shutdown
#  - address-family ipv6 unicast
#  - no redistribute direct route-map ospf102-direct-connect
#  - no redistribute eigrp 120 route-map rmap_1
#  - no area 0.0.0.100 filter-list route-map rmap_2 out
#  - no area 0.0.0.100 filter-list route-map rmap_1 in
#  - no area 0.0.0.100 range 2001:db2::/32
#  - no area 0.0.0.100 range 2001:db3::/32
#  - no vrf zone1
#  - no vrf zone2
#
# after:
#    processes:
#    - process_id: "102"
#      router_id: 198.51.100.1
#      shutdown: true
#      address_family:
#        afi: ipv6
#        safi: unicast
#    - process_id: "104"
#      router_id: 203.0.113.20

# After state:
# ------------
# nxos-9k-rdo# sh running-config | section "^router ospfv3"
# router ospfv3 102
#   router-id 198.51.100.1
#   address-family ipv6 unicast
#   shutdown
# router ospfv3 104
#   router-id 203.0.113.20

# Using deleted to delete a single OSPF process

# Before state:
# -------------
# nxos-9k-rdo# sh running-config | section "^router ospf .*"
# router ospfv3 100
#   router-id 203.0.113.20
# router ospfv3 102
#   router-id 198.51.100.1
#   address-family ipv6 unicast
#     redistribute direct route-map ospf102-direct-connect
#     redistribute eigrp 120 route-map rmap_1
#     area 0.0.0.100 filter-list route-map rmap_2 out
#     area 0.0.0.100 filter-list route-map rmap_1 in
#     area 0.0.0.100 range 2001:db2::/32 not-advertise
#     area 0.0.0.100 range 2001:db3::/32 cost 120
#   vrf zone1
#     router-id 198.51.100.129
#     area 0.0.0.102 nssa no-summary default-information-originate
#     area 0.0.0.103 nssa no-summary
#     area 0.0.0.103 nssa translate type7 always
#   vrf zone2
#     auto-cost reference-bandwidth 45 Gbps

- name: Delete a single OSPFv3 process
  cisco.nxos.nxos_ospfv3:
    config:
      processes:
      - process_id: 102
    state: deleted

# Task output:
# ------------
# before:
#    processes:
#    - process_id: "100"
#      router_id: 203.0.113.20
#    - address_family:
#        afi: ipv4
#        safi: unicast
#        areas:
#        - area_id: 0.0.0.100
#          filter_list:
#          - direction: out
#            route_map: rmap_2
#          - direction: in
#            route_map: rmap_1
#          ranges:
#          - not_advertise: true
#            prefix: 2001:db2::/32
#          - cost: 120
#            prefix: 2001:db3::/32
#        redistribute:
#        - protocol: direct
#          route_map: ospf102-direct-connect
#        - id: "120"
#          protocol: eigrp
#          route_map: rmap_1
#      process_id: "102"
#      router_id: 198.51.100.1
#      vrfs:
#      - areas:
#        - area_id: 0.0.0.102
#          nssa:
#            default_information_originate: true
#            no_summary: true
#        - area_id: 0.0.0.103
#          nssa:
#            no_summary: true
#            translate:
#              type7:
#                always: true
#        router_id: 198.51.100.129
#        vrf: zone1
#      - auto_cost:
#          reference_bandwidth: 45
#          unit: Gbps
#        vrf: zone2
#
# commands:
#  - no router ospfv3 102
#
# after:
#   processes:
#   - process_id: "100"
#     router_id: 203.0.113.20

# After state:
# ------------
# nxos-9k-rdo# sh running-config | section "^router ospfv3"
# router ospfv3 100
#   router-id 203.0.113.20

# Using deleted all OSPFv3 processes from the device

# Before state:
# -------------
# nxos-9k-rdo# sh running-config | section "^router ospfv3"
# router ospfv3 100
#   router-id 203.0.113.20
# router ospfv3 102
#   router-id 198.51.100.1
#   address-family ipv6 unicast
#     redistribute direct route-map ospf102-direct-connect
#     redistribute eigrp 120 route-map rmap_1
#     area 0.0.0.100 filter-list route-map rmap_2 out
#     area 0.0.0.100 filter-list route-map rmap_1 in
#     area 0.0.0.100 range 2001:db2::/32 not-advertise
#     area 0.0.0.100 range 2001:db3::/32 cost 120
#   vrf zone1
#     router-id 198.51.100.129
#     area 0.0.0.102 nssa no-summary default-information-originate
#     area 0.0.0.103 nssa no-summary
#     area 0.0.0.103 nssa translate type7 always
#   vrf zone2
#     auto-cost reference-bandwidth 45 Gbps

- name: Delete all OSPFv3 processes from the device
  cisco.nxos.nxos_ospfv3:
    state: deleted

# Task output:
# ------------
# before:
#    processes:
#    - process_id: "100"
#      router_id: 203.0.113.20
#    - address_family:
#        afi: ipv4
#        safi: unicast
#        areas:
#        - area_id: 0.0.0.100
#          filter_list:
#          - direction: out
#            route_map: rmap_2
#          - direction: in
#            route_map: rmap_1
#          ranges:
#          - not_advertise: true
#            prefix: 2001:db2::/32
#          - cost: 120
#            prefix: 2001:db3::/32
#        redistribute:
#        - protocol: direct
#          route_map: ospf102-direct-connect
#        - id: "120"
#          protocol: eigrp
#          route_map: rmap_1
#      process_id: "102"
#      router_id: 198.51.100.1
#      vrfs:
#      - areas:
#        - area_id: 0.0.0.102
#          nssa:
#            default_information_originate: true
#            no_summary: true
#        - area_id: 0.0.0.103
#          nssa:
#            no_summary: true
#            translate:
#              type7:
#                always: true
#        router_id: 198.51.100.129
#        vrf: zone1
#      - auto_cost:
#          reference_bandwidth: 45
#          unit: Gbps
#        vrf: zone2
#
# commands:
#  - no router ospfv3 100
#  - no router ospfv3 102
#
#  after: {}

# After state:
# ------------
# nxos-9k-rdo# sh running-config | section "^router ospfv3"
# nxos-9k-rdo#

# Using rendered

- name: Render platform specific configuration lines with state rendered (without connecting to the device)
  cisco.nxos.nxos_ospfv3:
    config:
      processes:
      - process_id: 100
        router_id: 203.0.113.20
      - process_id: 102
        router_id: 198.51.100.1
        address_family:
          afi: ipv6
          safi: unicast
          areas:
          - area_id: 0.0.0.100
            filter_list:
            - route_map: rmap_1
              direction: in
            - route_map: rmap_2
              direction: out
            ranges:
            - prefix: 2001:db2::/32
              not_advertise: true
            - prefix: 2001:db3::/32
              cost: 120
          redistribute:
          - protocol: eigrp
            id: 120
            route_map: rmap_1
          - protocol: direct
            route_map: ospf102-direct-connect
        vrfs:
        - vrf: zone1
          router_id: 198.51.100.129
          areas:
          - area_id: 0.0.0.102
            nssa:
              default_information_originate: true
              no_summary: true
          - area_id: 0.0.0.103
            nssa:
              no_summary: true
              translate:
                type7:
                  always: true
        - vrf: zone2
          auto_cost:
            reference_bandwidth: 45
            unit: Gbps
    state: rendered

# Task output:
# ------------
# rendered:
#  - router ospfv3 100
#  - router-id 203.0.113.20
#  - router ospfv3 102
#  - router-id 198.51.100.1
#  - address-family ipv6 unicast
#  - redistribute eigrp 120 route-map rmap_1
#  - redistribute direct route-map ospf102-direct-connect
#  - area 0.0.0.100 filter-list route-map rmap_1 in
#  - area 0.0.0.100 filter-list route-map rmap_2 out
#  - area 0.0.0.100 range 2001:db2::/32 not-advertise
#  - area 0.0.0.100 range 2001:db3::/32 cost 120
#  - vrf zone1
#  - router-id 198.51.100.129
#  - area 0.0.0.102 nssa no-summary default-information-originate
#  - area 0.0.0.103 nssa no-summary
#  - area 0.0.0.103 nssa translate type7 always
#  - vrf zone2
#  - auto-cost reference-bandwidth 45 Gbps

# Using parsed

# parsed.cfg
# ------------
# router ospfv3 100
#   router-id 192.0.100.1
#   address-family ipv6 unicast
#     redistribute direct route-map ospf-direct-connect
#     redistribute eigrp 120 route-map rmap_1
#     area 0.0.0.100 filter-list route-map rmap_2 out
#     area 0.0.0.100 filter-list route-map rmap_1 in
#     area 0.0.0.100 range 2001:db2::/32 not-advertise
#     area 0.0.0.100 range 2001:db3::/32 cost 120
#   vrf zone1
#     router-id 198.51.100.129
#     area 0.0.100.1 nssa no-summary no-redistribution
# router ospfv3 102
#   router-id 198.54.100.1
#   shutdown

- name: Parse externally provided OSPFv3 config
  cisco.nxos.nxos_ospfv3:
    running_config: "{{ lookup('file', 'ospfv2.cfg') }}"
    state: parsed

# Task output:
# ------------
# parsed:
#   processes:
#   - process_id: "100"
#     address_family:
#       afi: ipv6
#       safi: unicast
#       areas:
#       - area_id: 0.0.0.101
#         nssa:
#           no_redistribution: true
#           no_summary: true
#       - area_id: 0.0.0.102
#         stub:
#           no_summary: true
#         filter_list:
#           - direction: out
#             route_map: rmap_2
#           - direction: in
#             route_map: rmap_1
#         ranges:
#           - not_advertise: true
#             prefix: 192.0.2.0/24
#           - cost: 120
#             prefix: 192.0.3.0/24
#       redistribute:
#       - protocol: direct
#         route_map: ospf-direct-connect
#       - id: "120"
#         protocol: eigrp
#         route_map: rmap_1
#     router_id: 192.0.100.1
#     vrfs:
#       - vrf: zone1
#         areas:
#           - area_id: 0.0.100.1
#             nssa:
#               no_redistribution: true
#               no_summary: true
#         router_id: 192.0.100.2
#   - process_id: "102"
#     router_id: 198.54.100.1
#     shutdown: True

# Using gathered

- name: Gather OSPFv3 facts using gathered
  cisco.nxos.nxos_ospfv3:
    state: gathered

# Task output:
# ------------
#  gathered:
#    processes:
#    - process_id: "100"
#      router_id: 203.0.113.20
#    - address_family:
#        afi: ipv4
#        safi: unicast
#        areas:
#        - area_id: 0.0.0.100
#          filter_list:
#          - direction: out
#            route_map: rmap_2
#          - direction: in
#            route_map: rmap_1
#          ranges:
#          - not_advertise: true
#            prefix: 2001:db2::/32
#          - cost: 120
#            prefix: 2001:db3::/32
#        redistribute:
#        - protocol: direct
#          route_map: ospf102-direct-connect
#        - id: "120"
#          protocol: eigrp
#          route_map: rmap_1
#      process_id: "102"
#      router_id: 198.51.100.1
#      vrfs:
#      - areas:
#        - area_id: 0.0.0.102
#          nssa:
#            default_information_originate: true
#            no_summary: true
#        - area_id: 0.0.0.103
#          nssa:
#            no_summary: true
#            translate:
#              type7:
#                always: true
#        router_id: 198.51.100.129
#        vrf: zone1
#      - auto_cost:
#          reference_bandwidth: 45
#          unit: Gbps
#        vrf: zone2
#
"""
RETURN = """
before:
  description: The configuration prior to the model invocation.
  returned: always
  type: dict
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
after:
  description: The resulting configuration model invocation.
  returned: when changed
  type: dict
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
commands:
  description: The set of commands pushed to the remote device.
  returned: always
  type: list
  sample:
    - "router ospfv3 102"
    - "router-id 198.54.100.1"
    - "router ospfv3 100"
    - "router-id 192.0.100.1"
    - "address-family ipv6 unicast"
    - "redistribute eigrp 120 route-map rmap_1"
    - "redistribute direct route-map ospf-direct-connect"
    - "area 0.0.0.100 filter-list route-map rmap_1 in"
    - "area 0.0.0.100 filter-list route-map rmap_2 out"
    - "area 0.0.0.100 range 2001:db2::/32 not-advertise"
    - "area 0.0.0.100 range 2001:db3::/32 cost 120"
    - "vrf zone1"
    - "router-id 192.0.100.2"
    - "vrf zone2"
    - "auto-cost reference-bandwidth 45 Gbps"
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
    - "router ospfv3 102"
    - "router-id 198.54.100.1"
    - "router ospfv3 100"
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.argspec.ospfv3.ospfv3 import (
    Ospfv3Args,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.config.ospfv3.ospfv3 import (
    Ospfv3,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Ospfv3Args.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Ospfv3(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: elb_instance
version_added: 1.0.0
short_description: De-registers or registers instances from EC2 ELBs
description:
  - This module de-registers or registers an AWS EC2 instance from the ELBs
    that it belongs to.
  - Will be marked changed when called only if there are ELBs found to operate on.
author:
  - "John Jarvis (@jarv)"
options:
  state:
    description:
      - Register or deregister the instance.
    required: true
    choices: ['present', 'absent']
    type: str
  instance_id:
    description:
      - EC2 Instance ID.
    required: true
    type: str
  ec2_elbs:
    description:
      - List of ELB names
      - Required when I(state=present).
    type: list
    elements: str
  enable_availability_zone:
    description:
      - Whether to enable the availability zone of the instance on the target ELB if the availability zone has not already
        been enabled.
      - If I(enable_availability_zone=no), the task will fail if the availability zone is not enabled on the ELB.
    type: bool
    default: true
  wait:
    description:
      - Wait for instance registration or deregistration to complete successfully before returning.
    type: bool
    default: true
  wait_timeout:
    description:
      - Number of seconds to wait for an instance to change state.
      - If I(wait_timeout=0) then this module may return an error if a transient error occurs.
      - If non-zero then any transient errors are ignored until the timeout is reached.
      - Ignored when I(wait=no).
    default: 0
    type: int
notes:
  - The ec2_elbs fact previously set by this module was deprecated in release 2.1.0 and since release
    4.0.0 is no longer set.
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# basic pre_task and post_task example
pre_tasks:
  - name: Instance De-register
    community.aws.elb_instance:
      instance_id: "{{ ansible_ec2_instance_id }}"
      state: absent
    register: deregister_instances
    delegate_to: localhost
roles:
  - myrole
post_tasks:
  - name: Instance Register
    community.aws.elb_instance:
      instance_id: "{{ ansible_ec2_instance_id }}"
      ec2_elbs: "{{ deregister_instances.updated_elbs }}"
      state: present
    delegate_to: localhost
"""

RETURN = r"""
updated_elbs:
  description: A list of ELB names that the instance has been added to or removed from.
  returned: always
  type: list
  elements: str
"""

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


class ElbManager:
    """Handles EC2 instance ELB registration and de-registration"""

    def __init__(self, module, instance_id=None, ec2_elbs=None):
        retry_decorator = AWSRetry.jittered_backoff()
        self.module = module
        self.client_asg = module.client("autoscaling", retry_decorator=retry_decorator)
        self.client_ec2 = module.client("ec2", retry_decorator=retry_decorator)
        self.client_elb = module.client("elb", retry_decorator=retry_decorator)
        self.instance_id = instance_id
        self.lbs = self._get_instance_lbs(ec2_elbs)
        self.changed = False
        self.updated_elbs = set()

    def deregister(self, wait, timeout):
        """De-register the instance from all ELBs and wait for the ELB
        to report it out-of-service"""

        for lb in self.lbs:
            instance_ids = [i["InstanceId"] for i in lb["Instances"]]
            if self.instance_id not in instance_ids:
                continue

            self.updated_elbs.add(lb["LoadBalancerName"])

            if self.module.check_mode:
                self.changed = True
                continue

            try:
                self.client_elb.deregister_instances_from_load_balancer(
                    aws_retry=True,
                    LoadBalancerName=lb["LoadBalancerName"],
                    Instances=[{"InstanceId": self.instance_id}],
                )
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                self.module.fail_json_aws(
                    e, "Failed to deregister instance from load balancer", load_balancer=lb, instance=self.instance_id
                )

            # The ELB is changing state in some way. Either an instance that's
            # InService is moving to OutOfService, or an instance that's
            # already OutOfService is being deregistered.
            self.changed = True

        if wait:
            for lb in self.lbs:
                self._await_elb_instance_state(lb, "Deregistered", timeout)

    def register(self, wait, enable_availability_zone, timeout):
        """Register the instance for all ELBs and wait for the ELB
        to report the instance in-service"""
        for lb in self.lbs:
            instance_ids = [i["InstanceId"] for i in lb["Instances"]]
            if self.instance_id in instance_ids:
                continue

            self.updated_elbs.add(lb["LoadBalancerName"])

            if enable_availability_zone:
                self.changed |= self._enable_availailability_zone(lb)

            if self.module.check_mode:
                self.changed = True
                continue

            try:
                self.client_elb.register_instances_with_load_balancer(
                    aws_retry=True,
                    LoadBalancerName=lb["LoadBalancerName"],
                    Instances=[{"InstanceId": self.instance_id}],
                )
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                self.module.fail_json_aws(
                    e, "Failed to register instance with load balancer", load_balancer=lb, instance=self.instance_id
                )

            self.changed = True

        if wait:
            for lb in self.lbs:
                self._await_elb_instance_state(lb, "InService", timeout)

    @AWSRetry.jittered_backoff()
    def _describe_elbs(self, **params):
        paginator = self.client_elb.get_paginator("describe_load_balancers")
        results = paginator.paginate(**params).build_full_result()
        return results["LoadBalancerDescriptions"]

    def exists(self, lbtest):
        """Verify that the named ELB actually exists"""

        found = False
        for lb in self.lbs:
            if lb["LoadBalancerName"] == lbtest:
                found = True
                break
        return found

    def _enable_availailability_zone(self, lb):
        """Enable the current instance's availability zone in the provided lb.
        Returns True if the zone was enabled or False if no change was made.
        lb: load balancer"""
        instance = self._get_instance()
        desired_zone = instance["Placement"]["AvailabilityZone"]

        if desired_zone in lb["AvailabilityZones"]:
            return False

        if self.module.check_mode:
            return True

        try:
            self.client_elb.enable_availability_zones_for_load_balancer(
                aws_retry=True,
                LoadBalancerName=lb["LoadBalancerName"],
                AvailabilityZones=[desired_zone],
            )
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            self.module.fail_json_aws(e, "Failed to enable AZ on load balancers", load_balancer=lb, zone=desired_zone)

        return True

    def _await_elb_instance_state(self, lb, awaited_state, timeout):
        """Wait for an ELB to change state"""
        if self.module.check_mode:
            return

        initial_state = self._get_instance_health(lb)

        if awaited_state == initial_state:
            return

        if awaited_state == "InService":
            waiter = self.client_elb.get_waiter("instance_in_service")
        elif awaited_state == "Deregistered":
            waiter = self.client_elb.get_waiter("instance_deregistered")
        elif awaited_state == "OutOfService":
            waiter = self.client_elb.get_waiter("instance_deregistered")
        else:
            self.module.fail_json(msg="Could not wait for unknown state", awaited_state=awaited_state)

        try:
            waiter.wait(
                LoadBalancerName=lb["LoadBalancerName"],
                Instances=[{"InstanceId": self.instance_id}],
                WaiterConfig={"Delay": 1, "MaxAttempts": timeout},
            )
        except botocore.exceptions.WaiterError as e:
            self.module.fail_json_aws(
                e, msg="Timeout waiting for instance to reach desired state", awaited_state=awaited_state
            )
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            self.module.fail_json_aws(
                e, msg="Error while waiting for instance to reach desired state", awaited_state=awaited_state
            )

        return

    def _get_instance_health(self, lb):
        """
        Check instance health, should return status object or None under
        certain error conditions.
        """
        try:
            status = self.client_elb.describe_instance_health(
                aws_retry=True,
                LoadBalancerName=lb["LoadBalancerName"],
                Instances=[{"InstanceId": self.instance_id}],
            )["InstanceStates"]
        except is_boto3_error_code("InvalidInstance"):
            return None
        except (
            botocore.exceptions.ClientError,
            botocore.exceptions.BotoCoreError,
        ) as e:  # pylint: disable=duplicate-except
            self.module.fail_json_aws(e, msg="Failed to get instance health")

        if not status:
            return None

        return status[0]["State"]

    def _get_instance_lbs(self, ec2_elbs=None):
        """Returns a list of ELBs attached to self.instance_id
        ec2_elbs: an optional list of elb names that will be used
                  for elb lookup instead of returning what elbs
                  are attached to self.instance_id"""

        list_params = dict()
        if not ec2_elbs:
            ec2_elbs = self._get_auto_scaling_group_lbs()

        if ec2_elbs:
            list_params["LoadBalancerNames"] = ec2_elbs

        try:
            elbs = self._describe_elbs(**list_params)
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            self.module.fail_json_aws(e, "Failed to describe load balancers")

        if ec2_elbs:
            return elbs

        # If ec2_elbs wasn't specified, then filter out LBs we're not a member
        # of.
        lbs = []
        for lb in elbs:
            instance_ids = [i["InstanceId"] for i in lb["Instances"]]
            if self.instance_id in instance_ids:
                lbs.append(lb)

        return lbs

    def _get_auto_scaling_group_lbs(self):
        """Returns a list of ELBs associated with self.instance_id
        indirectly through its auto scaling group membership"""

        try:
            asg_instances = self.client_asg.describe_auto_scaling_instances(
                aws_retry=True, InstanceIds=[self.instance_id]
            )["AutoScalingInstances"]
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            self.module.fail_json_aws(e, msg="Failed to describe ASG Instance")

        if len(asg_instances) > 1:
            self.module.fail_json(msg="Illegal state, expected one auto scaling group instance.")

        if not asg_instances:
            # Instance isn't a member of an ASG
            return []

        asg_name = asg_instances[0]["AutoScalingGroupName"]

        try:
            asg_instances = self.client_asg.describe_auto_scaling_groups(
                aws_retry=True, AutoScalingGroupNames=[asg_name]
            )["AutoScalingGroups"]
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            self.module.fail_json_aws(e, msg="Failed to describe ASG Instance")

        if len(asg_instances) != 1:
            self.module.fail_json(msg="Illegal state, expected one auto scaling group.")

        return asg_instances[0]["LoadBalancerNames"]

    def _get_instance(self):
        """Returns the description of an instance"""
        try:
            result = self.client_ec2.describe_instances(aws_retry=True, InstanceIds=[self.instance_id])
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            self.module.fail_json_aws(e, msg="Failed to describe ASG Instance")
        return result["Reservations"][0]["Instances"][0]


def main():
    argument_spec = dict(
        state={"required": True, "choices": ["present", "absent"]},
        instance_id={"required": True},
        ec2_elbs={"default": None, "required": False, "type": "list", "elements": "str"},
        enable_availability_zone={"default": True, "required": False, "type": "bool"},
        wait={"required": False, "default": True, "type": "bool"},
        wait_timeout={"required": False, "default": 0, "type": "int"},
    )
    required_if = [
        ("state", "present", ["ec2_elbs"]),
    ]

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        required_if=required_if,
        supports_check_mode=True,
    )

    ec2_elbs = module.params["ec2_elbs"]
    wait = module.params["wait"]
    enable_availability_zone = module.params["enable_availability_zone"]
    timeout = module.params["wait_timeout"]
    instance_id = module.params["instance_id"]

    elb_man = ElbManager(module, instance_id, ec2_elbs)

    if ec2_elbs is not None:
        for elb in ec2_elbs:
            if not elb_man.exists(elb):
                module.fail_json(msg=f"ELB {elb} does not exist")

    if module.params["state"] == "present":
        elb_man.register(wait, enable_availability_zone, timeout)
    elif module.params["state"] == "absent":
        elb_man.deregister(wait, timeout)

    module.exit_json(
        changed=elb_man.changed,
        updated_elbs=list(elb_man.updated_elbs),
    )


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Tim Cragg (@timcragg) <timcragg@cisco.com>
# Copyright: (c) 2023, Akini Ross (@akinross) <akinross@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_system_endpoint_controls
short_description: Manage System Endpoint Controls (ep:IpAgingP, ep:ControlP, and ep:LoopProtectP)
description:
- Manage System Endpoint Controls on Cisco ACI fabrics.
options:
  ip_aging:
    description: Configuration container for IP Aging.
    type: dict
    suboptions:
      admin_state:
        description:
        - Whether to enable IP Aging Controls on the fabric.
        type: bool
  roque_ep_control:
    description: Configuration container for Rogue EP Control.
    type: dict
    suboptions:
      admin_state:
        description:
        - Whether to enable Rogue EP Control on the fabric.
        type: bool
      interval:
        description:
        - The rogue endpoint detection interval in seconds.
        type: int
      multiplication_factor:
        description:
        - The rogue endpoint detection multiplication factor.
        type: int
      hold_interval:
        description:
        - The rogue endpoint hold interval in seconds.
        type: int
  ep_loop_protection:
    description: Configuration container for EP Loop Protection.
    type: dict
    suboptions:
      admin_state:
        description:
        - Whether to enable EP Loop Protection on the fabric.
        type: bool
      interval:
        description:
        - The loop protection detection interval in seconds.
        type: int
      multiplication_factor:
        description:
        - The loop protection detection multiplication factor.
        type: int
      action:
        description:
        - The action(s) to take when a loop is detected.
        type: list
        elements: str
        choices: [ bd, port ]
  state:
    description:
    - Use C(present) for updating configuration.
    - Use C(query) for showing current configuration.
    type: str
    choices: [ present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(ep:IpAgingP), B(ep:ControlP), and B(ep:LoopProtectP).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Tim Cragg (@timcragg)
- Akini Ross (@akinross)
"""

EXAMPLES = r"""
- name: Set system endpoint controls settings
  cisco.aci.aci_system_endpoint_controls:
    host: apic
    username: admin
    password: SomeSecretPassword
    admin_state: true
    ip_aging:
      admin_state: true
    roque_ep_control:
      admin_state: true
      interval: 50
      multiplication_factor: 10
      hold_interval: 2000
    ep_loop_protection:
      admin_state: true
      interval: 70
      multiplication_factor: 15
      action: [ bd, port ]
  delegate_to: localhost

- name: Query system endpoint controls settings
  cisco.aci.aci_system_endpoint_controls:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result

"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec
from ansible_collections.cisco.aci.plugins.module_utils.constants import EP_LOOP_PROTECTION_ACTION_MAPPING


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        ip_aging=dict(type="dict", options=dict(admin_state=dict(type="bool"))),
        roque_ep_control=dict(
            type="dict",
            options=dict(
                admin_state=dict(type="bool"),
                interval=dict(type="int"),
                multiplication_factor=dict(type="int"),
                hold_interval=dict(type="int"),
            ),
        ),
        ep_loop_protection=dict(
            type="dict",
            options=dict(
                admin_state=dict(type="bool"),
                interval=dict(type="int"),
                multiplication_factor=dict(type="int"),
                action=dict(type="list", elements="str", choices=["bd", "port"]),
            ),
        ),
        state=dict(type="str", default="present", choices=["present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[["state", "present", ["ip_aging", "roque_ep_control", "ep_loop_protection"], True]],
    )

    aci = ACIModule(module)
    state = module.params.get("state")

    aci.construct_url(
        root_class=dict(
            aci_class="infraInfra",
            aci_rn="infra",
        ),
        child_classes=["epIpAgingP", "epControlP", "epLoopProtectP"],
    )

    aci.get_existing()

    if state == "present":
        child_configs = []
        ip_aging = module.params.get("ip_aging")
        roque_ep_control = module.params.get("roque_ep_control")
        ep_loop_protection = module.params.get("ep_loop_protection")

        if ip_aging:
            child_configs.append(
                {"epIpAgingP": {"attributes": {"name": "default", "adminSt": aci.boolean(ip_aging.get("admin_state"), "enabled", "disabled")}}}
            )

        if roque_ep_control:
            child_configs.append(
                {
                    "epControlP": {
                        "attributes": {
                            "name": "default",
                            "adminSt": aci.boolean(roque_ep_control.get("admin_state"), "enabled", "disabled"),
                            "rogueEpDetectIntvl": roque_ep_control.get("interval"),
                            "rogueEpDetectMult": roque_ep_control.get("multiplication_factor"),
                            "holdIntvl": roque_ep_control.get("hold_interval"),
                        }
                    }
                }
            )

        if ep_loop_protection:
            actions = None
            if ep_loop_protection.get("action"):
                actions = ",".join(sorted([EP_LOOP_PROTECTION_ACTION_MAPPING.get(action) for action in ep_loop_protection.get("action")]))
            child_configs.append(
                {
                    "epLoopProtectP": {
                        "attributes": {
                            "name": "default",
                            "adminSt": aci.boolean(ep_loop_protection.get("admin_state"), "enabled", "disabled"),
                            "loopDetectIntvl": ep_loop_protection.get("interval"),
                            "loopDetectMult": ep_loop_protection.get("multiplication_factor"),
                            "action": actions,
                        }
                    }
                }
            )

        aci.payload(
            aci_class="infraInfra",
            class_config=dict(),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="infraInfra")

        aci.post_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2021, Milan Zink <zeten30@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: efs_tag
version_added: 2.0.0
short_description: create and remove tags on Amazon EFS resources
description:
  - Creates and removes tags for Amazon EFS resources.
  - Resources are referenced by their ID (filesystem or filesystem access point).
author:
  - Milan Zink (@zeten30)
options:
  resource:
    description:
      - EFS Filesystem ID or EFS Filesystem Access Point ID.
    type: str
    required: True
  state:
    description:
      - Whether the tags should be present or absent on the resource.
    default: present
    choices: ['present', 'absent']
    type: str
  tags:
    description:
      - A dictionary of tags to add or remove from the resource.
      - If the value provided for a tag is null and I(state=absent), the tag will be removed regardless of its current value.
    type: dict
    required: True
    aliases: ['resource_tags']
  purge_tags:
    description:
      - Whether unspecified tags should be removed from the resource.
      - Note that when combined with I(state=absent), specified tags with non-matching values are not purged.
    type: bool
    default: false
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Ensure tags are present on a resource
  community.aws.efs_tag:
    resource: fs-123456ab
    state: present
    tags:
      Name: MyEFS
      Env: Production

- name: Remove the Env tag if it's currently 'development'
  community.aws.efs_tag:
    resource: fsap-78945ff
    state: absent
    tags:
      Env: development

- name: Remove all tags except for Name
  community.aws.efs_tag:
    resource: fsap-78945ff
    state: absent
    tags:
      Name: foo
    purge_tags: true

- name: Remove all tags
  community.aws.efs_tag:
    resource: fsap-78945ff
    state: absent
    tags: {}
    purge_tags: true
"""

RETURN = r"""
tags:
  description: A dict containing the tags on the resource
  returned: always
  type: dict
added_tags:
  description: A dict of tags that were added to the resource
  returned: If tags were added
  type: dict
removed_tags:
  description: A dict of tags that were removed from the resource
  returned: If tags were removed
  type: dict
"""

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
except ImportError:
    # Handled by AnsibleAWSModule
    pass

from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import ansible_dict_to_boto3_tag_list
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.tagging import compare_aws_tags

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule

MAX_AWS_RETRIES = 10  # How many retries to perform when an API call is failing
WAIT_RETRY = 5  # how many seconds to wait between propagation status polls


def get_tags(efs, module, resource):
    """
    Get resource tags
    """
    try:
        return boto3_tag_list_to_ansible_dict(efs.list_tags_for_resource(aws_retry=True, ResourceId=resource)["Tags"])
    except (BotoCoreError, ClientError) as get_tags_error:
        module.fail_json_aws(get_tags_error, msg=f"Failed to fetch tags for resource {resource}")


def main():
    """
    MAIN
    """
    argument_spec = dict(
        resource=dict(required=True),
        tags=dict(type="dict", required=True, aliases=["resource_tags"]),
        purge_tags=dict(type="bool", default=False),
        state=dict(default="present", choices=["present", "absent"]),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)
    resource = module.params["resource"]
    tags = module.params["tags"]
    state = module.params["state"]
    purge_tags = module.params["purge_tags"]

    result = {"changed": False}

    efs = module.client("efs", retry_decorator=AWSRetry.jittered_backoff())

    current_tags = get_tags(efs, module, resource)

    add_tags, remove = compare_aws_tags(current_tags, tags, purge_tags=purge_tags)

    remove_tags = {}

    if state == "absent":
        for key in tags:
            if key in current_tags and (tags[key] is None or current_tags[key] == tags[key]):
                remove_tags[key] = current_tags[key]

    for key in remove:
        remove_tags[key] = current_tags[key]

    if remove_tags:
        result["changed"] = True
        result["removed_tags"] = remove_tags
        if not module.check_mode:
            try:
                efs.untag_resource(aws_retry=True, ResourceId=resource, TagKeys=list(remove_tags.keys()))
            except (BotoCoreError, ClientError) as remove_tag_error:
                module.fail_json_aws(
                    remove_tag_error, msg=f"Failed to remove tags {remove_tags} from resource {resource}"
                )

    if state == "present" and add_tags:
        result["changed"] = True
        result["added_tags"] = add_tags
        current_tags.update(add_tags)
        if not module.check_mode:
            try:
                tags = ansible_dict_to_boto3_tag_list(add_tags)
                efs.tag_resource(aws_retry=True, ResourceId=resource, Tags=tags)
            except (BotoCoreError, ClientError) as set_tag_error:
                module.fail_json_aws(set_tag_error, msg=f"Failed to set tags {add_tags} on resource {resource}")

    result["tags"] = get_tags(efs, module, resource)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: wireless_accespoint_configuration
short_description: Resource module for Wireless Accespoint Configuration
description:
- Manage operation create of the resource Wireless Accespoint Configuration.
- User can configure multiple access points with required options using this intent API.
version_added: '6.7.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  adminStatus:
    description: Configure the access point's admin status. Set this parameter's value
      to "true" to enable it and "false" to disable it.
    type: bool
  apHeight:
    description: Configure the height of the access point by setting a value between
      3 and height of the floor.
    type: int
  apList:
    description: Wireless Accespoint Configuration's apList.
    elements: dict
    suboptions:
      apName:
        description: The current host name of the access point.
        type: str
      apNameNew:
        description: The modified hostname of the access point.
        type: str
      macAddress:
        description: The ethernet MAC address of the access point.
        type: str
    type: list
  apMode:
    description: Configure the access point's mode for local/flexconnect mode, set "0";
      for monitor mode, set "1"; for sniffer mode, set "4"; and for bridge/flex+bridge
      mode, set "5".
    type: int
  configureAdminStatus:
    description: To change the access point's admin status, set this parameter's value
      to "true".
    type: bool
  configureApHeight:
    description: To change the access point's height, set this parameter's value to
      "true".
    type: bool
  configureApMode:
    description: To change the access point's mode, set this parameter's value to "true".
    type: bool
  configureFailoverPriority:
    description: To change the access point's failover priority, set this parameter's
      value to "true".
    type: bool
  configureHAController:
    description: To change the access point's HA controller, set this parameter's value
      to "true".
    type: bool
  configureLedBrightnessLevel:
    description: To change the access point's LED brightness level, set this parameter's
      value to "true".
    type: bool
  configureLedStatus:
    description: To change the access point's LED status, set this parameter's value
      to "true".
    type: bool
  configureLocation:
    description: To change the access point's location, set this parameter's value to
      "true".
    type: bool
  failoverPriority:
    description: Configure the acess point's failover priority for low, set "1"; for
      medium, set "2"; for high, set "3"; and for critical, set "4".
    type: int
  ledBrightnessLevel:
    description: Configure the access point's LED brightness level by setting a value
      between 1 and 8.
    type: int
  ledStatus:
    description: Configure the access point's LED status. Set "true" to enable its status
      and "false" to disable it.
    type: bool
  location:
    description: Configure the access point's location.
    type: str
  primaryControllerName:
    description: Configure the hostname for an access point's primary controller.
    type: str
  primaryIpAddress:
    description: Wireless Accespoint Configuration's primaryIpAddress.
    suboptions:
      address:
        description: Configure the IP address for an access point's primary controller.
        type: str
    type: dict
  radioConfigurations:
    description: Wireless Accespoint Configuration's radioConfigurations.
    elements: dict
    suboptions:
      adminStatus:
        description: Configure the admin status on the specified radio for an access
          point. Set this parameter's value to "true" to enable it and "false" to disable
          it.
        type: bool
      antennaCableName:
        description: Configure the antenna cable name on the specified radio for an
          access point. If cable loss needs to be configured, set this parameter's value
          to "other".
        type: str
      antennaDegree:
        description: Configure the antenna degree on the specified radio for an access
          point.
        type: int
      antennaElevAngleDegree:
        description: Configure the antenna elevation angle on the specified radio for
          an access point.
        type: int
      antennaElevAngleSign:
        description: Configure the antenna elevation angle direction on the specified
          radio for an access point for up, set "1"; for down, set "-1".
        type: int
      antennaGain:
        description: Configure the antenna gain on the specified radio for an access
          point by setting a decimal value (in dBi).
        type: int
      antennaPatternName:
        description: Configure the antenna pattern name on the specified radio for an
          access point. If antenna gain needs to be configured, set this parameter's
          value to "other".
        type: str
      cableLoss:
        description: Configure the cable loss on the specified radio for an access point
          by setting a decimal value (in dBi).
        type: int
      channelAssignmentMode:
        description: Configure the channel assignment mode on the specified radio for
          an access point for global mode, set "1"; and for custom mode, set "2".
        type: int
      channelNumber:
        description: Configure the channel number on the specified radio for an access
          point.
        type: int
      channelWidth:
        description: Configure the channel width on the specified radio for an access
          point for 20 MHz, set "3"; for 40 MHz, set "4"; for 80 MHz, set "5"; and for
          160 MHz, set "6".
        type: int
      cleanAirSI:
        description: Configure CleanAir or Spectrum Intelligence on the specified radio
          for an access point. Set this parameter's value to "0" to disable the feature
          or "1" to enable it.
        type: int
      configureAdminStatus:
        description: To change the admin status on the specified radio for an access
          point, set this parameter's value to "true".
        type: bool
      configureAntennaCable:
        description: To change the antenna cable name on the specified radio for an
          access point, set this parameter's value to "true".
        type: bool
      configureAntennaDegree:
        description: To change the antenna degree on the specified radio for an access
          point, set this parameter's value to "true".
        type: bool
      configureAntennaPatternName:
        description: To change the antenna pattern name on the specified radio for an
          access point, set the value for this parameter to "true".
        type: bool
      configureChannel:
        description: To change the channel on the specified radio for an access point,
          set this parameter's value to "true".
        type: bool
      configureChannelWidth:
        description: To change the channel width on the specified radio for an access
          point, set this parameter's value to "true".
        type: bool
      configureCleanAirSI:
        description: To enable or disable either CleanAir or Spectrum Intelligence on
          the specified radio for an access point, set this parameter's value to "true".
        type: bool
      configureElevAngleDegree:
        description: To change the elevation angle degree on the specified radio for
          an access point, set this parameter's value to "true".
        type: bool
      configurePower:
        description: To change the power assignment mode on the specified radio for
          an access point, set this parameter's value to "true".
        type: bool
      configureRadioRoleAssignment:
        description: To change the radio role on the specified radio for an access point,
          set this parameter's value to "true".
        type: bool
      powerAssignmentMode:
        description: Configure the power assignment mode on the specified radio for
          an access point for global mode, set "1"; and for custom mode, set "2".
        type: int
      powerlevel:
        description: Configure the power level on the specified radio for an access
          point by setting a value between 1 and 8.
        type: int
      radioBand:
        description: Configure the band on the specified radio for an access point for
          2.4 GHz, set "RADIO24"; for 5 GHz, set "RADIO5".
        type: str
      radioRoleAssignment:
        description: Configure one of the following roles on the specified radio for
          an access point "auto", "serving", or "monitor".
        type: str
      radioType:
        description: Configure an access point's radio band for 2.4 GHz, set "1"; for
          5 GHz, set "2"; for XOR, set "3"; and for 6 GHz, set "6".
        type: int
    type: list
  secondaryControllerName:
    description: Configure the hostname for an access point's secondary controller.
    type: str
  secondaryIpAddress:
    description: Wireless Accespoint Configuration's secondaryIpAddress.
    suboptions:
      address:
        description: Configure the IP address for an access point's secondary controller.
        type: str
    type: dict
  tertiaryControllerName:
    description: Configure the hostname for an access point's tertiary controller.
    type: str
  tertiaryIpAddress:
    description: Wireless Accespoint Configuration's tertiaryIpAddress.
    suboptions:
      address:
        description: Configure the IP address for an access point's tertiary controller.
        type: str
    type: dict
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Wireless ConfigureAccessPoints
  description: Complete reference of the ConfigureAccessPoints API.
  link: https://developer.cisco.com/docs/dna-center/#!configure-access-points
notes:
  - SDK Method used are
    wireless.Wireless.configure_access_points,

  - Paths used are
    post /dna/intent/api/v1/wireless/accesspoint-configuration,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.wireless_accespoint_configuration:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    adminStatus: true
    apHeight: 0
    apList:
    - apName: string
      apNameNew: string
      macAddress: string
    apMode: 0
    configureAdminStatus: true
    configureApHeight: true
    configureApMode: true
    configureFailoverPriority: true
    configureHAController: true
    configureLedBrightnessLevel: true
    configureLedStatus: true
    configureLocation: true
    failoverPriority: 0
    ledBrightnessLevel: 0
    ledStatus: true
    location: string
    primaryControllerName: string
    primaryIpAddress:
      address: string
    radioConfigurations:
    - adminStatus: true
      antennaCableName: string
      antennaDegree: 0
      antennaElevAngleDegree: 0
      antennaElevAngleSign: 0
      antennaGain: 0
      antennaPatternName: string
      cableLoss: 0
      channelAssignmentMode: 0
      channelNumber: 0
      channelWidth: 0
      cleanAirSI: 0
      configureAdminStatus: true
      configureAntennaCable: true
      configureAntennaDegree: true
      configureAntennaPatternName: true
      configureChannel: true
      configureChannelWidth: true
      configureCleanAirSI: true
      configureElevAngleDegree: true
      configurePower: true
      configureRadioRoleAssignment: true
      powerAssignmentMode: 0
      powerlevel: 0
      radioBand: string
      radioRoleAssignment: string
      radioType: 0
    secondaryControllerName: string
    secondaryIpAddress:
      address: string
    tertiaryControllerName: string
    tertiaryIpAddress:
      address: string

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



# -*- coding: utf-8 -*-

# Copyright (c) 2021, Andrew Pantuso (@ajpantuso) <ajpantuso@gmail.com>
# Copyright (c) 2018, Dag Wieers (@dagwieers) <dag@wieers.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
  name: from_csv
  short_description: Converts CSV text input into list of dicts
  version_added: 2.3.0
  author: Andrew Pantuso (@Ajpantuso)
  description:
    - Converts CSV text input into list of dictionaries.
  options:
    _input:
      description: A string containing a CSV document.
      type: string
      required: true
    dialect:
      description:
        - The CSV dialect to use when parsing the CSV file.
        - Possible values include V(excel), V(excel-tab) or V(unix).
      type: str
      default: excel
    fieldnames:
      description:
        - A list of field names for every column.
        - This is needed if the CSV does not have a header.
      type: list
      elements: str
    delimiter:
      description:
        - A one-character string used to separate fields.
        - When using this parameter, you change the default value used by O(dialect).
        - The default value depends on the dialect used.
      type: str
    skipinitialspace:
      description:
        - Whether to ignore any whitespaces immediately following the delimiter.
        - When using this parameter, you change the default value used by O(dialect).
        - The default value depends on the dialect used.
      type: bool
    strict:
      description:
        - Whether to raise an exception on bad CSV input.
        - When using this parameter, you change the default value used by O(dialect).
        - The default value depends on the dialect used.
      type: bool
'''

EXAMPLES = '''
- name: Parse a CSV file's contents
  ansible.builtin.debug:
    msg: >-
      {{ csv_data | community.general.from_csv(dialect='unix') }}
  vars:
    csv_data: |
      Column 1,Value
      foo,23
      bar,42
  # Produces the following list of dictionaries:
  #   {
  #     "Column 1": "foo",
  #     "Value": "23",
  #   },
  #   {
  #     "Column 1": "bar",
  #     "Value": "42",
  #   }
'''

RETURN = '''
  _value:
    description: A list with one dictionary per row.
    type: list
    elements: dictionary
'''

from ansible.errors import AnsibleFilterError
from ansible.module_utils.common.text.converters import to_native

from ansible_collections.community.general.plugins.module_utils.csv import (initialize_dialect, read_csv, CSVError,
                                                                            DialectNotAvailableError,
                                                                            CustomDialectFailureError)


def from_csv(data, dialect='excel', fieldnames=None, delimiter=None, skipinitialspace=None, strict=None):

    dialect_params = {
        "delimiter": delimiter,
        "skipinitialspace": skipinitialspace,
        "strict": strict,
    }

    try:
        dialect = initialize_dialect(dialect, **dialect_params)
    except (CustomDialectFailureError, DialectNotAvailableError) as e:
        raise AnsibleFilterError(to_native(e))

    reader = read_csv(data, dialect, fieldnames)

    data_list = []

    try:
        for row in reader:
            data_list.append(row)
    except CSVError as e:
        raise AnsibleFilterError("Unable to process file: %s" % to_native(e))

    return data_list


class FilterModule(object):

    def filters(self):
        return {
            'from_csv': from_csv
        }
